import sys

from PySide6 import QtGui, QtWidgets

from .chat_window import ChatWindow


def main():
    app = QtWidgets.QApplication(sys.argv)

    # Set application-wide font
    font = QtGui.QFont("Arial", 10)
    app.setFont(font)

    window = ChatWindow()
    window.show()
    app.lastWindowClosed.connect(window.save_all)
    sys.exit(app.exec())



import ollama
from PySide6 import QtCore
from . import CONTEXT_MARGIN, CHARACTERS_PER_TOKEN
from .message_widget import MessageWidget
import logging

logger = logging.getLogger(__name__)


class Generator(QtCore.QThread):
    interrupt: bool = False
    nextChunk = QtCore.Signal(str)
    error = QtCore.Signal(str)

    def __init__(
        self,
        model: str,
        temperature: float,
        prompt: str,
        widgets: list[MessageWidget],
        context_size: int,
    ):
        super().__init__()
        self.model = model
        self.prompt = prompt or "You are a helpful AI assistant."
        self.temperature = temperature
        self.widgets = widgets
        self.context_size = context_size

    def run(self):
        # enable endless chatting by clipping the part of the conversation
        # that the llm can see, but keep the system prompt at all times
        conversation_window = []
        num_token = len(self.prompt) / CHARACTERS_PER_TOKEN
        for w in reversed(self.widgets):
            num_token += len(w.content) / CHARACTERS_PER_TOKEN
            if num_token > self.context_size - CONTEXT_MARGIN:
                break
            conversation_window.append(w.dict())
        conversation_window.append({"role": "system", "content": self.prompt})
        conversation_window.reverse()

        try:
            for response in ollama.chat(
                model=self.model,
                messages=conversation_window,
                stream=True,
                options={"temperature": self.temperature},
            ):
                if self.interrupt:
                    return
                chunk = response["message"]["content"]
                self.nextChunk.emit(chunk)
        except Exception as e:
            error_message = f"""Error generating response: {str(e)}\n\n
Please make sure that the model '{self.model}' is available.
You can run 'ollama run {self.model}' in terminal to check."""
            self.error.emit(error_message)



import torch
from transformers import AutoModelForCausalLM, AutoTokenizer, pipeline

torch.random.manual_seed(0)
model_id = "microsoft/Phi-3-small-8k-instruct"
model = AutoModelForCausalLM.from_pretrained(
    model_id, 
    torch_dtype="auto", 
    trust_remote_code=True, 
)
assert torch.cuda.is_available(), "This model needs a GPU to run ..."
device = torch.cuda.current_device()
model = model.to(device)
tokenizer = AutoTokenizer.from_pretrained(model_id)

messages = [
    {"role": "user", "content": "Can you provide ways to eat combinations of bananas and dragonfruits?"},
    {"role": "assistant", "content": "Sure! Here are some ways to eat bananas and dragonfruits together: 1. Banana and dragonfruit smoothie: Blend bananas and dragonfruits together with some milk and honey. 2. Banana and dragonfruit salad: Mix sliced bananas and dragonfruits together with some lemon juice and honey."},
    {"role": "user", "content": "What about solving an 2x + 3 = 7 equation?"},
]

pipe = pipeline(
    "text-generation",
    model=model,
    tokenizer=tokenizer,
    device=device
)

generation_args = {
    "max_new_tokens": 500,
    "return_full_text": False,
    "temperature": 0.0,
    "do_sample": False,
}

output = pipe(messages, **generation_args)
print(output[0]['generated_text'])


from PySide6 import QtWidgets, QtCore
from mistune import html


class MessageWidget(QtWidgets.QLabel):
    role: str
    content: str

    def __init__(self, role: str, text: str, *, parent=None):
        super().__init__(parent)

        self.setWordWrap(True)
        self.setFrameStyle(QtWidgets.QFrame.Shape.NoFrame)
        self.setTextInteractionFlags(
            QtCore.Qt.TextInteractionFlag.TextSelectableByMouse
        )

        if role == "user":
            self.setStyleSheet("""
                QFrame {
                    background-color: #F5F5F5;
                    border-radius: 10px;
                    margin: 5px 5px 5px 50px;
                    text-align: right;
                }
            """)
        else:
            self.setStyleSheet("""
                QFrame {
                    background-color: #E3F2FD;
                    border-radius: 10px;
                    margin: 5px 50px 5px 5px;
                }
            """)

        self.role = role
        self.set_text(text)  # this sets content

    def set_text(self, text):
        self.content = text
        if not text:
            if self.role == "assistant":
                self.setText("<em>Thinking...</em>")
            else:
                self.hide()
        else:
            self.setText(html(text))

    def add_text(self, text):
        self.content += text
        self.setText(html(self.content))

    def dict(self):
        return {"role": self.role, "content": self.content}



from PySide6 import QtWidgets


class CharacterBar(QtWidgets.QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)

        size_policy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed
        )
        self.setSizePolicy(size_policy)

        self.config_button = QtWidgets.QPushButton("Configure")
        self.character_selector = QtWidgets.QComboBox()
        self.new_button = QtWidgets.QPushButton("New character")
        self.clipboard_button = QtWidgets.QPushButton("Clipboard")
        self.summary_button = QtWidgets.QPushButton("Summary")
        self.num_token = QtWidgets.QLabel()

        layout = QtWidgets.QHBoxLayout()
        layout.addWidget(self.config_button)
        layout.addWidget(self.character_selector)
        layout.addWidget(self.new_button)
        layout.addWidget(self.clipboard_button)
        layout.addWidget(self.summary_button)
        layout.addWidget(self.num_token)
        self.setLayout(layout)

    def set_character_manually(self, names: list[str], new_name: str):
        if new_name not in names:
            names.append(new_name)
        names.sort()
        self.character_selector.blockSignals(True)
        self.character_selector.clear()
        for name in names:
            self.character_selector.addItem(name)
        self.character_selector.setCurrentText(new_name)
        self.character_selector.blockSignals(False)

    def current_character(self):
        return self.character_selector.currentText()

    def update_num_token(self, num: int, size: int):
        if num < 0:
            text = ""
        else:
            k = 1024
            text = f"{num/k:.1f} (est) | {size/k:.0f} k token"
        self.num_token.setText(text)



from typing import Any, Dict, List


def chat(
    model: str,
    messages: List[str] = [],
    stream: bool = False,
    options: Dict[str, Any] = {},
):
    if stream:
        for _ in range(1):
            yield {"message": {"content": "dummy content"}}
    else:
        return {"message": {"content": "dummy content"}}



from PySide6 import QtWidgets
import ollama
from .data_models import Character, Memory


class ConfigDialog(QtWidgets.QDialog):
    def __init__(self, character: Character, memory: Memory, *, parent=None):
        super().__init__(parent)

        self.memory = memory

        self.setWindowTitle("Character configuration")
        self.setMinimumWidth(500)

        self.name = QtWidgets.QLineEdit()
        self.name.setText(character.name)

        self.prompt = QtWidgets.QTextEdit()
        self.prompt.setPlainText(character.prompt)
        self.prompt.setAcceptRichText(False)
        self.prompt.setMinimumHeight(200)

        self.model = QtWidgets.QComboBox()
        installed_models = []
        for item in ollama.list()["models"]:
            installed_models.append(item["name"])
        installed_models.sort()
        self.model.addItems(installed_models)
        i = self.model.findText(character.model)
        if i >= 0:
            self.model.setCurrentIndex(i)
        else:
            self.model.addItem(character.model)

        self.temperature = QtWidgets.QDoubleSpinBox()
        self.temperature.setRange(0, 2.0)
        self.temperature.setDecimals(1)
        self.temperature.setValue(character.temperature)

        self.save_conversation = QtWidgets.QCheckBox()
        self.save_conversation.setChecked(character.save_conversation)

        self.clear_conversation = QtWidgets.QCheckBox()
        self.clear_conversation.setChecked(False)

        self.delete_character = QtWidgets.QCheckBox()

        clayout = QtWidgets.QFormLayout()
        clayout.addRow("Name", self.name)
        clayout.addRow("Prompt", self.prompt)
        clayout.addRow("Model", self.model)
        clayout.addRow("Temperature", self.temperature)
        clayout.addRow("Save conversation", self.save_conversation)
        clayout.addRow("Clear conversation", self.clear_conversation)
        clayout.addRow("Delete character", self.delete_character)

        vlayout = QtWidgets.QVBoxLayout()
        vlayout.addLayout(clayout)

        # Dialog buttons
        button_box = QtWidgets.QDialogButtonBox(
            QtWidgets.QDialogButtonBox.StandardButton.Ok
            | QtWidgets.QDialogButtonBox.StandardButton.Cancel
        )
        button_box.accepted.connect(self.accept)
        button_box.rejected.connect(self.reject)

        # Main layout
        vlayout.addWidget(button_box)
        self.setLayout(vlayout)

    def result(self):
        if self.delete_character.isChecked():
            return self.name.text()
        else:
            memory = self.memory
            if self.clear_conversation.isChecked():
                memory.messages = []

            return (
                Character(
                    name=self.name.text(),
                    prompt=self.prompt.toPlainText(),
                    model=self.model.currentText(),
                    temperature=self.temperature.value(),
                    save_conversation=self.save_conversation.isChecked(),
                ),
                memory,
            )



from . import CHARACTERS_PER_TOKEN
from .data_models import Message
import ollama
from pydantic import BaseModel
import logging
from typing import TypeVar

T = TypeVar("T", bound=BaseModel)

logger = logging.getLogger(__name__)


def estimate_num_tokens(prompt: str, messages: list[Message]):
    # estimate number of token
    num_char = len(prompt)
    for m in messages:
        num_char += len(m.content)
    return num_char / CHARACTERS_PER_TOKEN


def generate_json_response(
    model: str, prompt: str, data_model: T, *, trials: int = 5
) -> T | None:
    for trial in range(trials):
        response: str = ollama.generate(
            model=model, prompt=prompt, options={"temperature": 0.1}
        )["response"]
        logger.info(f"Raw response (trial={trial}):\n{response}")
        # clip comments before or after the json
        try:
            clipped = response[response.index("{") : response.rindex("}") + 1]
            return data_model.model_validate(clipped)
        except Exception as e:
            logger.error("JSON parsing failed (trial={trial}):", e)
    return None



def parse(text):
    """
    Convert text containing markdown-style italic markers (*) and newlines to HTML.

    Args:
        text (str): Input text

    Returns:
        str: HTML code
    """
    parts = text.split("*")

    result = []
    for i, part in enumerate(parts):
        if not part or part.isspace():
            continue
        if i % 2 == 0:
            # Even indices are normal text
            result.append(part)
        else:
            # Odd indices are italic text
            result.append(f"<i>{part}</i>")

    return "".join(result).replace("\n", "<br>")



from plaitime.parser import parse
import pytest


@pytest.mark.parametrize(
    "input, expected",
    [
        ("Hello *world*!", "Hello <i>world</i>!"),
        ("*All italic*", "<i>All italic</i>"),
        ("No italics here", "No italics here"),
        ("*First* and *second* italic", "<i>First</i> and <i>second</i> italic"),
        (
            "Text with *multiple* words in *italic* style",
            "Text with <i>multiple</i> words in <i>italic</i> style",
        ),
        # Edge cases
        ("Unclosed asterisk*", "Unclosed asterisk"),
        ("**", ""),  # Double asterisk
        ("", ""),  # Empty string
        ("*Unclosed asterisk", "<i>Unclosed asterisk</i>"),
    ],
)
def test_parse(input, expected):
    got = parse(input)
    assert got == expected



from pydantic import BaseModel, PositiveFloat


class Character(BaseModel):
    name: str = "Assistant"
    prompt: str = ""
    model: str = "llama3.2"
    temperature: PositiveFloat = 0.7
    save_conversation: bool = True


class Message(BaseModel):
    role: str
    content: str


class Memory(BaseModel):
    messages: list[Message] = []


class Config(BaseModel):
    current_character: str = ""



from pydantic import BaseModel
import logging
from pathlib import Path
from typing import TypeVar
from datetime import datetime

import ollama
from PySide6 import QtCore, QtGui, QtWidgets

from . import (
    CHARACTER_DIRECTORY,
    CONFIG_FILE_NAME,
    MEMORY_DIRECTORY,
    STORY_EXTRACTION_PROMPT,
)
from .character_bar import CharacterBar
from .config_dialog import ConfigDialog
from .data_models import Character, Config, Memory, Message
from .generator import Generator
from .message_widget import MessageWidget
from .util import estimate_num_tokens

T = TypeVar("T", bound=BaseModel)

logging.basicConfig(level=logging.INFO)

logger = logging.getLogger(__name__)


class InputBox(QtWidgets.QTextEdit):
    sendMessage = QtCore.Signal()

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setAcceptRichText(False)

    def keyPressEvent(self, event):
        mod = event.modifiers()
        match event.key():
            case QtCore.Qt.Key_Return:
                if not (mod & QtCore.Qt.KeyboardModifier.ShiftModifier):
                    self.sendMessage.emit()
                    return
        super().keyPressEvent(event)


class ChatWindow(QtWidgets.QMainWindow):
    character: Character
    generator: Generator | None

    def __init__(self):
        super().__init__()
        self.character = Character()
        self.generator = None

        self.setWindowTitle("PAI")
        self.setMinimumSize(600, 500)

        # Create top bar
        self.character_bar = CharacterBar()
        self.setMenuWidget(self.character_bar)
        self.character_bar.config_button.clicked.connect(self.show_config_dialog)
        self.character_bar.new_button.clicked.connect(self.new_character)
        self.character_bar.summary_button.clicked.connect(self.generate_summary)
        self.character_bar.character_selector.currentTextChanged.connect(
            self.switch_character
        )
        self.character_bar.clipboard_button.clicked.connect(self.copy_to_clipboard)

        # Create central widget and layout
        central_widget = QtWidgets.QWidget()
        self.setCentralWidget(central_widget)
        layout = QtWidgets.QVBoxLayout(central_widget)

        # Create scroll area for messages
        self.scroll_area = QtWidgets.QScrollArea()
        self.scroll_area.setWidgetResizable(True)
        self.scroll_area.setHorizontalScrollBarPolicy(
            QtCore.Qt.ScrollBarPolicy.ScrollBarAlwaysOff
        )
        self.vscrollbar = self.scroll_area.verticalScrollBar()
        self.vscrollbar.rangeChanged.connect(self.scroll_to_bottom)
        layout.addWidget(self.scroll_area)

        # Create input area
        self.input_box = InputBox()
        self.input_box.setMaximumHeight(100)
        self.input_box.setPlaceholderText("Type your message here...")
        self.input_box.sendMessage.connect(self.send_message_and_generate_response)

        layout.addWidget(self.input_box)

        # Create send button
        self.send_button = QtWidgets.QPushButton("Send")
        self.send_button.clicked.connect(self.send_message_and_generate_response)
        layout.addWidget(self.send_button)

        # Must be at the end
        config = self.load_config()
        self.load_character(config.current_character)

    def load_messages(self, prompt: str, memory: Memory):
        self.messages_widget = QtWidgets.QWidget()
        self.messages_layout = QtWidgets.QVBoxLayout(self.messages_widget)
        self.messages_layout.addStretch()
        self.scroll_area.setWidget(self.messages_widget)

        widgets = []
        n = len(memory.messages)
        for i, m in enumerate(memory.messages):
            if i < n - 1 or m.role == "assistant":
                w = self.add_message(m.role, m.content)
                widgets.append(w)
            else:
                self.input_box.setText(m.content)

        num_token = estimate_num_tokens(prompt, widgets)
        self.character_bar.update_num_token(num_token, self.context_size)

    def load_config(self) -> Config:
        return load(CONFIG_FILE_NAME, Config)

    def save_config(self):
        config = Config(current_character=self.character_bar.current_character())
        save(config, CONFIG_FILE_NAME)

    def load_character(self, name: str):
        if not name:
            # load any character
            for fname in CHARACTER_DIRECTORY.glob("*.json"):
                name = fname.stem
                break
        self.character = load(CHARACTER_DIRECTORY / f"{name}.json", Character)
        names = get_character_names()
        self.character_bar.set_character_manually(names, self.character.name)
        self.context_size = get_context_size(self.character.model)
        memory = load(MEMORY_DIRECTORY / f"{name}.json", Memory)
        self.load_messages(self.character.prompt, memory)

    def save_character(self):
        c = self.character
        logger.info(f"saving character {c.name}")
        save(c, CHARACTER_DIRECTORY / f"{c.name}.json")

        if c.save_conversation:
            widgets = self.get_message_widgets()
            messages = [Message.model_validate(w.dict()) for w in widgets]
            user_text = self.get_user_text()
            if user_text:
                messages.append(Message(role="user", content=user_text))
            if messages:
                memory = Memory(messages=messages)
                save(memory, MEMORY_DIRECTORY / f"{c.name}.json")
                return
        # if nothing shall be saved if there is nothing to save, remove the file
        path = MEMORY_DIRECTORY / f"{c.name}.json"
        path.unlink(missing_ok=True)

    def delete_character(self, name: str):
        logger.info(f"deleting character {name}")
        for path in (
            CHARACTER_DIRECTORY / f"{name}.json",
            MEMORY_DIRECTORY / f"{name}.json",
        ):
            path.unlink(missing_ok=True)

    def save_all(self):
        self.save_config()
        self.save_character()

    def show_config_dialog(self):
        widgets = self.get_message_widgets()
        messages = []
        for w in widgets:
            messages.append(Message.model_validate(w.dict()))
        self.configure_character(self.character, Memory(messages=messages))

    def configure_character(self, character, memory):
        dialog = ConfigDialog(character, memory, parent=self)
        if dialog.exec() == QtWidgets.QDialog.DialogCode.Accepted:
            r: str | tuple[Character, Memory] = dialog.result()
            if isinstance(r, str):
                self.delete_character(r)
                self.load_character("")
            else:
                character, memory = r
                if self.character.name != character.name:
                    self.delete_character(self.character.name)
                self.character = character
                names = get_character_names()
                self.character_bar.set_character_manually(names, character.name)
                self.context_size = get_context_size(self.character.model)
                self.load_messages(character.prompt, memory)

    def new_character(self):
        self.save_character()
        self.configure_character(Character(), Memory())
        names = get_character_names()
        self.character_bar.set_character_manually(names, self.character.name)

    def switch_character(self, name):
        logger.info(f"switching character to {name}")
        self.save_character()
        self.load_character(name)

    def add_message(self, role: str, text: str) -> MessageWidget:
        message = MessageWidget(role, text)
        self.messages_layout.addWidget(message)
        return message

    def undo_last_response(self):
        messages = self.get_message_widgets()
        if self.generator and self.generator.isRunning():
            assert len(messages) >= 2
            self.generator.interrupt = True
            self.generator.wait()
        else:
            if not messages:
                return
        assistant_message = messages.pop()
        assert assistant_message.role == "assistant"
        user_message = messages.pop()
        assert user_message.role == "user"
        self.input_box.setText(user_message.content)
        for m in (assistant_message, user_message):
            m.setParent(None)
            m.deleteLater()

    def scroll_to_bottom(self, _, vmax):
        self.vscrollbar.setValue(vmax)

    def get_user_text(self):
        return self.input_box.toPlainText().strip()

    def send_message_and_generate_response(self):
        user_text = self.get_user_text()
        self.input_box.clear()
        self.send_button.setEnabled(False)
        self.input_box.setEnabled(False)
        self.add_message("user", user_text)
        self.generate_response()

    def generator_finished(self):
        self.send_button.setEnabled(True)
        self.input_box.setEnabled(True)
        self.input_box.setFocus()
        self.generator = None

    def generate_response(self):
        widgets = self.get_message_widgets()
        self.generator = Generator(
            self.character.model,
            self.character.temperature,
            self.character.prompt,
            widgets,
            self.context_size,
        )
        mw = self.add_message("assistant", "")
        self.generator.nextChunk.connect(mw.add_text)
        self.generator.error.connect(mw.set_text)
        self.generator.finished.connect(self.generator_finished)
        self.generator.start()

    def copy_to_clipboard(self):
        messages = self.get_message_widgets()
        clipboard = QtGui.QGuiApplication.clipboard()
        clipboard.setText("\n\n".join(m.content for m in messages))

    def get_message_widgets(self) -> list[MessageWidget]:
        return [
            child
            for child in self.messages_widget.children()
            if isinstance(child, MessageWidget)
        ]

    def generate_summary(self):
        messages = self.get_message_widgets()
        text = self.character.prompt + "\n\n".join(m.content for m in messages)

        self.input_box.setEnabled(False)
        self.send_button.setEnabled(False)
        response = ollama.generate(
            model=self.character.model,
            prompt=STORY_EXTRACTION_PROMPT.format(text),
            options={"temperature": 0.1},
        )["response"]

        text = self.input_box.toPlainText()
        self.input_box.setPlainText(text + response)
        self.input_box.setEnabled(True)
        self.send_button.setEnabled(True)
        self.input_box.setFocus()

    def keyPressEvent(self, event):
        key = event.key()
        if key == QtCore.Qt.Key_Escape:
            self.undo_last_response()
        else:
            super().keyPressEvent(event)


def get_context_size(model):
    d = ollama.show(model)["model_info"]
    for key in d:
        if "context_length" in key:
            return d[key]
    raise RuntimeError("context_length not found")


def save(obj: BaseModel, filename: Path):
    backup = None
    if filename.exists() and isinstance(obj, Memory):
        st = filename.stat()
        suffix = datetime.fromtimestamp(st.st_mtime).strftime(r"%Y-%m-%d-%H-%M-%S")
        backup = filename.rename(filename.with_suffix(f".backup-{suffix}"))
    with open(filename, "wb") as f:
        c1 = obj.model_dump_json(indent=4).encode("utf-8")
        f.write(c1)
    if backup:
        with open(backup, "rb") as f:
            c2 = f.read()
        if c1 == c2:
            backup.unlink()


def load(filename: Path, cls: T) -> T:
    if filename.exists():
        try:
            with open(filename, "rb") as f:
                return cls.model_validate_json(f.read())
        except Exception as e:
            logger.error(e)

        if cls is Memory:
            import json

            messages = []
            try:
                with open(filename, "rb") as f:
                    data = json.load(f)
                for m in data["messages"]:
                    messages.append(Message(role=m["role"], content=m["content"]))
                return Memory(messages=messages)
            except Exception as e:
                logger.error(e)

    logger.info(f"{filename} does not exist")
    return cls()


def get_character_names():
    names = []
    for fname in CHARACTER_DIRECTORY.glob("*.json"):
        names.append(fname.stem)
    return names



from pathlib import Path

BASE_DIRECTORY = Path.home() / ".plaitime"
BASE_DIRECTORY.mkdir(exist_ok=True)

CONFIG_FILE_NAME = BASE_DIRECTORY / "config.json"
CHARACTER_DIRECTORY = BASE_DIRECTORY / "characters"
CHARACTER_DIRECTORY.mkdir(exist_ok=True)
MEMORY_DIRECTORY = BASE_DIRECTORY / "memories"
MEMORY_DIRECTORY.mkdir(exist_ok=True)

CHARACTERS_PER_TOKEN = 4  # on average
CONTEXT_MARGIN = 512

STORY_EXTRACTION_PROMPT = """
You are a professional author.

Analyze the story excerpt and extract facts from the story, so that another author can continue the story faithfully.
Try to isolate facts which are established and are of long-term importance to continue the story. Ignore transient or fleeting information.

As you are reading the story excerpt, focus on these questions:

* Character descriptions
  - Start by listing all the characters of the story.
  - How old is the character?
  - Physical appearance? (any details about face or body?)
  - How are they dressed?
  - What is their occupation?
  - Unusual mannerisms?
  - What is revealed about their backstory?

* Relationships between characters
  - Are the characters friends or enemies, close or distant?
  - Are they flirting or in love? Is the love one-sided?
  - Do the characters know each other well or not?

* World-building details
  - When does the story happen?
  - Atmosphere & genre?
  - List all places that are visited in the story and give a short description

# Story excerpt

{0}

# Response instructions

Remember: Try to isolate facts which are established and are of long-term importance to continue the story. Ignore transient or fleeting information.

Remember: Your task was to focus on these questions:

* Character descriptions
  - Start by listing all the characters of the story.
  - How old is the character?
  - Physical appearance? (any details about face or body?)
  - How are they dressed?
  - What is their occupation?
  - Unusual mannerisms?
  - What is revealed about their backstory?

* Relationships between characters
  - Are the characters friends or enemies, close or distant?
  - Are they flirting or in love? Is the love one-sided?
  - Do the characters know each other well or not?

* World-building details
  - When does the story happen?
  - Atmosphere & genre?
  - List all places that are visited in the story and give a short description

Return the information concisely in Markdown format:

**Character descriptions**

- Character 1
  - Info about character 1
- Character 2
  - Info about character 2
- ...

**Character relationships**

- Character 1 and 2: their relationship
- ...

**World-building details**

- Timeframe
- Genre & atmosphere
- Important places

**Summary of the story**
"""
